"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require('source-map-support');
const lodash_1 = __importDefault(require("lodash"));
const bluebird_1 = __importDefault(require("bluebird"));
const qrcode_1 = __importDefault(require("qrcode"));
const pngjs_1 = require("pngjs");
const gifencoder_1 = __importDefault(require("gifencoder"));
const omggif_1 = require("omggif");
const jsqr_1 = __importDefault(require("jsqr"));
const pako_1 = __importDefault(require("pako"));
/**
 * Convert data to a dancing QR code (animated GIF).
 *
 * @param data - Data to convert.
 * @param opts - Options for conversion.
 */
function encode(data, { bytesPerFrame = 400, delayPerFrame = 500 } = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        // Split string up in to frames.
        const buf = lodash_1.default.isString(data) ? Buffer.from(data) : data;
        const compress = pako_1.default.deflate(buf);
        const chunks = lodash_1.default.chunk(compress, bytesPerFrame).map(a => Buffer.from(a));
        const colors = [
            '#00aa11ff',
            '#00bb11ff',
            '#00cc11ff',
            '#00dd11ff',
            '#00ee11ff',
            '#00dd11ff',
            '#00cc11ff',
            '#00bb11ff',
        ];
        // Generate a QR image per frame.
        let frameCount = 0;
        let width = 0;
        let height = 0;
        const pngs = yield bluebird_1.default.map(chunks, (c) => __awaiter(this, void 0, void 0, function* () {
            // add in a header and pad everything.
            const header = Buffer.from([
                0x02, 0x26,
                0x00,
                0x00,
                0x00, 0x00,
            ]);
            header.writeUInt8(frameCount++, 2);
            header.writeUInt8(chunks.length, 3);
            header.writeUInt16BE(c.length, 4);
            const padding = Buffer.alloc(bytesPerFrame - c.length, 0);
            const data = Buffer.concat([header, c, padding]);
            let x = [{ data: data.toString('binary'), mode: 'byte' }];
            const png = pngjs_1.PNG.sync.read(yield qrcode_1.default.toBuffer(x));
            if (png.width > width) {
                width = png.width;
            }
            if (png.height > height) {
                height = png.height;
            }
            return png;
        }));
        // Encode in to an animated GIF.
        const encoder = new gifencoder_1.default(width, height);
        encoder.start();
        encoder.setRepeat(0); // Loop infinitely.
        encoder.setDelay(delayPerFrame);
        for (let i = 0; i < pngs.length; ++i) {
            const png = pngs[i];
            const dst = new pngjs_1.PNG({ width, height });
            dst.data.fill(255, 0, dst.data.length);
            pngjs_1.PNG.bitblt(png, dst, 0, 0, png.width, png.height, (width - png.width) / 2, (height - png.height) / 2);
            encoder.addFrame(dst.data);
        }
        // Profit.
        encoder.finish();
        return { gif: encoder.out.getData(), frameCount };
    });
}
exports.encode = encode;
/**
 * Convert dancing QR code (animated GIF) to data.
 *
 * @param image - Animated GIF to convert.
 */
function decode(image) {
    return __awaiter(this, void 0, void 0, function* () {
        const gif = new omggif_1.GifReader(image);
        let frames = [];
        let totalFrames = undefined;
        for (let i = 0; i < gif.numFrames(); ++i) {
            const { width, height } = gif.frameInfo(i);
            const buffer = new Array(width * height * 4);
            gif.decodeAndBlitFrameRGBA(i, buffer);
            const code = jsqr_1.default(new Uint8ClampedArray(buffer), width, height);
            const data = Buffer.from(code.data, 'binary');
            const magic = data.readUInt16BE(0);
            if (magic != 0x0226) {
                // Bad frame.
                continue;
            }
            const chunk = data.readUInt8(2);
            const total = data.readUInt8(3);
            const length = data.readUInt16BE(4);
            if (totalFrames === undefined) {
                totalFrames = total;
            }
            else if (totalFrames !== total) {
                throw new Error('Inconsistent totalFrames');
            }
            if (length > data.length - 6) {
                throw new Error('Data length too large');
            }
            frames[chunk] = data.slice(6, 6 + length);
        }
        for (let i = 0; i < totalFrames; ++i) {
            if (frames[i] === undefined) {
                throw new Error('Missing frame');
            }
        }
        const res = Buffer.concat(frames);
        return Buffer.from(pako_1.default.inflate(res));
    });
}
exports.decode = decode;
//# sourceMappingURL=index.js.map